%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%@Course      : Master Thesis Spring 2018
%@Task        : Closed-loop model predictive controller. Solves open-loop
%               optimization problem periodically and adds random
%               disturbance to the inputs
%@input       : none
%@output      : none

%@author      : Julie Berge Ims
%@organization: Department of Chemical Engineering, NTNU, Norway
%@created     : February 2018
%@requires    : MATLAB R2016a (not tested in other releases)                                                          
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear
clc

% Provide path to Casadi installation
addpath('/Users/juliebergeims/downloads/casadi-matlabR2015a-v3.0.0')
import casadi.*

%% Parameterization of open-loop

% Initial values for the differential states
dae_x0  = struct('compressor',struct('h',0.01)                         ,...
                 'clock',struct('time',0.0));  
                              
% Initial process input
u_comp = 0.85;                                   % Compressor speed [input]
u_choke = 0.565;                                 % Choke opening    [input]

% Number of time steps for prediction horizon 
N  = 20;   

% CVaR parameters
alpha = 0.001;                                  % Confidence level for risk 

% Uncertain parameter realizations
scenparam = [0.9 1 1.1];
%% Closed-lopp simulation 

run OL_main
%close all

% DAE-struct for integration
int_dae = struct('x', dae.x,                                            ...
                 'z', dae.z,                                            ...
                 'p',[dae.p ; dae.s],                                   ...
                 'ode', dae.ode,                                        ...
                 'alg', dae.alg );
                         
% Integrator using DAE-integrator "idas"
DAE_integrator = integrator('integrator',                               ...
                            'idas',int_dae, struct('t0',0,'tf',0.05)); 
                                         
% Integrating the DAE-struc
sol = DAE_integrator('x0', res_X(1,:)',                                 ...
                     'z0', res_Z(1,:)',                                 ...
                      'p', [res_U(1,:),scenparam(3) , 1]');

% Obtaining initial measured state(X) and first control input (U)
x_temp(1,:)       = sol.xf.full()';
u_temp_comp(1,:)  = input_comp(:);
u_temp_choke(1,:) = input_choke(:);

% Repeat process with shrinking horizon
for w = 2:1:20
    
    % Using first control input from the OL opt
    u_comp = res_U(1,1);
    u_choke= res_U(1,2);
    
    % Updating initial conditions 
    dae_x0.compressor.h = x_temp(w-1,1) + rand(1,1)*x_temp(w-1,1)/10;
    dae_x0.clock.time   = x_temp(w-1,2) + rand(1,1)*x_temp(w-1,2)/10;
        
    % OL opt by decreasing the prediction horizon by on time step 
    N = N-1; 
    run OL_main 
    close all
    
    % Integr. the DAE-struct
    sol = DAE_integrator('x0',sol.xf.full(),                            ...
                         'z0',sol.zf.full(),                            ...
                         'p', [res_U(1,:),scenparam(3), 1]');
    
    % Obtaining the CL states and inputs
    x_temp(w,:) = sol.xf.full()';
    u_temp_comp(w,:)  = input_comp(:);
    u_temp_choke(w,:) = input_choke(:);
end