%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%@Course      : Master Thesis Spring 2018
%@Task        : Open-loop optimization to obtain optimal control policy for
%             : the subsea system.
%@input       : none
%@output      : none

%@author      : Adriaen Verheyleweghen
%@modified    : Julie Berge Ims
%@organization: Department of Chemical Engineering, NTNU, Norway
%@requires    : MATLAB R2016a (not tested in other releases)     
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Provide path to Casadi installation
addpath('/Users/juliebergeims/downloads/casadi-matlabR2015a-v3.0.0')
import casadi.*

% Creating a model for optimization
[dae_x,dae_z,dae_u,~,dae_z0,dae_u0,dae_p,dae_ode,dae_alg] =             ...
                                      subsea_model(u_choke, u_comp);    
                                         
% Size of time step
dt = 0.05;

% Vector with time steps
hlist = repmat(dt,1,N);                 

% Sum of all time steps
tf = sum(hlist); 

%% Variable declarations

% Stochastic variables: GVF and degradation speed
dae_stoc.a     = dae_p.a;
dae_stoc.GVF   = dae_z.stream1.GVF;
dae_z.stream1  = rmfield(dae_z.stream1,'GVF');
dae_z0.stream1 = rmfield(dae_z0.stream1,'GVF');             

                       
% CVaR variables
CVaR  = struct( 'diff',MX.sym('diff'),                                  ...
                'var',MX.sym('var'),                                    ...
                'cvar',MX.sym('cvar')                                   ...
                );

% Profit variable
Profit = struct('profit',MX.sym('profit'));

% RUL variables
RUL  = struct('lambda',MX.sym('lambda'),                                ...      
                 'K',MX.sym('K')                                        ...                   
             );
         
% Scale parameter
la = 5.7;
lb = 1.2;
lc = 2.8;
lambda = @(h) la - lb*h - lc*sqrt(h) ;         

% Shape parameter
ka = 4.55;
kb = 0.1;
kc = 0.1;
K   = @(h)    ka + kb*h + kc*(h)^2 ;            

% Weibull quantile function
var = @(lambda,k,alpha) (lambda*(-log(1-alpha))^(1/k));

% Differential variables
dae_x.clock = struct('time',MX.sym('time'));
      
% ODE expressions
dae_ode=[dae_ode 1];  

% Algebraic variables
dae_z.RUL     =  struct('lambda',RUL.lambda,                             ...
                        'K',RUL.K);
                   
dae_z.CVaR    =  struct('var',CVaR.var,                                  ...
                        'cvar',CVaR.cvar);
                   
dae_z.Profit  =  struct('profit',Profit.profit);

% Algebraic initial conditions
dae_z0.RUL    =  struct('lambda',la,                                     ...
                        'K',ka);
                   
dae_z0.CVaR   =  struct('var',1,                                         ...
                        'cvar',0.8); 
                   
dae_z0.Profit = struct('profit',0.08);                

% Algebraic expressions
 dae_alg = [dae_alg;                                                    ...
            dae_z.RUL.lambda    - lambda(dae_x.compressor.h);           ...
            dae_z.RUL.K         - K(dae_x.compressor.h);                ...
            dae_z.CVaR.var      - var(lambda(dae_x.compressor.h),       ...
                                  K(dae_x.compressor.h),alpha);         ...
            dae_z.CVaR.cvar     - cvar_func(lambda(dae_x.compressor.h), ...
                                  K(dae_x.compressor.h),alpha);         ...
            dae_z.Profit.profit - (dae_z.stream4.mgdot)*(dt*(1+0.015)^  ...
                                  (-60*dae_x.clock.time))               ...
           ];

%% Optimal Control Problem set-up

% DAE-struct
dae      = struct;
dae.x    = casadi_struct2vec(dae_x);           
dae.z    = casadi_struct2vec(dae_z);           
dae.p    = casadi_struct2vec(dae_u);           
dae.s    = casadi_struct2vec(dae_stoc);       
dae.ode  = casadi_struct2vec(dae_ode);         
dae.alg  = dae_alg;   

% Objective function  
dae.quad = ( -dae_z.Profit.profit - dae_z.CVaR.cvar*0.025)';
       
%% Direct Collocation set-up

% Number of variables
nx = size(dae.x,1);                         
nu = size(dae.p,1);                            
nz = size(dae.z,1);               
ns = size(dae.s,1);                 

% Degree of interpolating polynomial
d = 3;

% Obtain collocation points of specific order and scheme. 
tau_root = casadi.collocation_points(d, 'radau');

% Obtain a function for collocation
collfun = simpleColl(dae,tau_root);                 
nlp = {};

%% Scenario-based collocation

% Three scenarios, based on "one" uncertain parameter, a
for scen=1:3
    
    % Symbolic primitive with given dimensions nx
    X0s = MX.sym('X0',nx);
    
    % Cell array [N+1,1] of empty matrices. Symbolic primitives 
    Xs = cell(N+1,1);
    for i=1:N+1
        Xs{i} = MX.sym(['X_' num2str(i)],nx);
    end
    
    % Cell array [N,1] of empty matrices.Symbolic primitives with for-loop
    XCs = cell(N,1);
    Zs  = cell(N,1);
    Us  = cell(N,1);
    Ss  = cell(N,1);
    
    for i=1:N                                         
        XCs{i} = MX.sym(['XC_' num2str(i)],nx,d);     
        Zs{i}  = MX.sym(['Z_'  num2str(i)],nz,d);     
        Us{i}  = MX.sym(['U_'  num2str(i)],nu,1);     
        Ss{i}  = MX.sym(['S_'  num2str(i)],ns,1);     
    end

    V_block = struct();
    V_block.X  = Sparsity.dense(nx,1);
    V_block.XC = Sparsity.dense(nx,d);
    V_block.Z  = Sparsity.dense(nz,d);
    V_block.U  = Sparsity.dense(nu,1);

    % Bounds on states and constraints
    lbx = {};
    ubx = {};
    lbg = {};
    ubg = {};

    % Objective function
    f = 0;

    % List of constraints
    g = {};

    % List of all decision variables (determines ordering)
    V    = {};
 %% Define bounds 

    % Bounds at 0<t<tf  
    x_lb_k = casadi_vec(dae_x, -inf);
    x_ub_k = casadi_vec(dae_x,  inf);
    u_lb_k = casadi_vec(dae_u, 0,'N',0.75,'Z',0);
    u_ub_k = casadi_vec(dae_u, inf,'N',1.05,'Z',1);
    z_lb_k = casadi_vec(dae_z, -inf,'srg',0,'stw',0,                    ...
                                    'stream4',{'P',1.5});
    z_ub_k = casadi_vec(dae_z, inf);
    
    % repeat for each collocation point
    z_lb_k = repmat(z_lb_k',d)';       
    z_ub_k = repmat(z_ub_k',d)';
    
    % Gather all bounds at 0<t<tf 
    lbx_k  = {casadi_vec(V_block,  0,'X',x_lb_k,'U',u_lb_k,'Z',z_lb_k)};
    ubx_k  = {casadi_vec(V_block,inf,'X',x_ub_k,'U',u_ub_k,'Z',z_ub_k)};

    % Bounds at t=tf 
    x_lb_tf = {x_lb_k};
    x_ub_tf = {x_ub_k};         

    % Initial guess
    guess       = [];
    x_guess     = casadi_struct2vec(dae_x0);
    z_guess     = casadi_struct2vec(dae_z0);
    u_guess     = casadi_struct2vec(dae_u0);
       
    for k=1:N

        % Add decision variables  xc: collocation points
        V = [V {casadi_vec(V_block,'X',Xs{k},'XC',XCs{k},               ...
                                   'Z',Zs{k},'U',Us{k})}];
        
        % Vector with inital guess                      
        guess =[guess;[repmat(x_guess,d+1,1);repmat(z_guess,d,1);u_guess]];                               

        lbx = [lbx lbx_k];
        ubx = [ubx ubx_k];

        if k==1
            tmp = {Xs{k}-X0s};
            g   = [g tmp]; 
            lbg = [lbg {zeros(size(tmp{:}))}];
            ubg = [ubg {zeros(size(tmp{:}))}];

            % Enforce nonanticipativity
            if scen==1
                U1 = Us{1};
            else
                tmp = {Us{k}-U1};
                g = [g tmp];
                lbg = [lbg {zeros(size(tmp{:}))}];
                ubg = [ubg {zeros(size(tmp{:}))}];
            end
        end
        
        % Obtain collocation expressions
        coll_out = collfun.call({hlist(k),Xs{k},XCs{k},Zs{k},Us{k},Ss{k}});

        tmp = coll_out(2);
        g   = [g tmp];                                    % System dynamics
        lbg = [lbg {zeros(size(tmp{:}))}];
        ubg = [ubg {zeros(size(tmp{:}))}];

        tmp = coll_out(3);
        g   = [g tmp];                              % Algebraic constraints
        lbg = [lbg {zeros(size(tmp{:}))}];
        ubg = [ubg {zeros(size(tmp{:}))}];

        tmp = {Xs{k+1}-coll_out{1}};
        g   = [g tmp];                            % Gap closing constraints
        lbg = [lbg {zeros(size(tmp{:}))}];
        ubg = [ubg {zeros(size(tmp{:}))}];
        
        % Cost function
        f = f + coll_out{4};
    end
   
    % Add final x to decision variables 
    V     = [V     , Xs(end)];
    guess = [guess ; 1;1];      

    % Bounds for final t
    lbx = [lbx x_lb_tf];
    ubx = [ubx x_ub_tf];
 
    %% Define the NLP
    nlp{scen} = struct('x',vertcat(V{:}),                               ...
                       'f',f,                                           ...
                       'g',vertcat(g{:}),                               ...
                       'p',vertcat(X0s,Ss{:})                           ...
                       );
end         

% Generate the full-space NLP by combining the individual scenario problems
nlp_full_space = nlp{1};
for i = 2:scen
    nlp_full_space.x = horzcat(nlp_full_space.x,nlp{i}.x);
    nlp_full_space.g = vertcat(nlp_full_space.g,nlp{i}.g);
    nlp_full_space.p = vertcat(nlp_full_space.p,nlp{i}.p);
    nlp_full_space.f = nlp_full_space.f + nlp{i}.f;
end

nlpfun = Function('nlp',nlp_full_space,char('x','p'),char('f','g'));

opts = struct('warn_initial_bounds',false,                              ...
              'gather_stats',true,                                      ...
              'print_time',false,                                       ...
              'ipopt',struct('linear_solver','mumps',                   ...
                             'max_iter',5E2,                            ...
                             'warm_start_init_point','yes',             ...
                             'mu_init',1E-5,                            ...
                             'replace_bounds','yes',                    ...
                             'print_level',5,                           ...
                             'tol',1E-8)                                ...                           
              ); 
          
% Creating nlp sovler for the nlp_full_space system with the options 'opts'       
solver = nlpsol('solver','ipopt',nlp_full_space,opts);

% Scenario realizations
b1 = [scenparam(1),1];
b2 = [scenparam(2),1];
b3 = [scenparam(3) ,1];

% Scenarios
s11 = [b1;repmat(b1,N-1,1)]';
s22 = [b2;repmat(b2,N-1,1)]';
s33 = [b3;repmat(b3,N-1,1)]';

%% Solve the NLP

x0 = casadi_struct2vec(dae_x0);
x0_orig= repmat(guess(:),1,scen);

res = solver('x0',x0_orig,                                              ... 
             'p',[full(x0);s11(:);full(x0);s22(:);full(x0);s33(:)],     ...
             'lbg',zeros(size(nlp_full_space.g)),                       ...
             'ubg',zeros(size(nlp_full_space.g)),                       ...
             'lbx',repmat(vertcat(lbx{:}),1,scen),                      ...
             'ubx',repmat(vertcat(ubx{:}),1,scen)                       ...
             ); 

%% Plotting

close all
figure('units','normalized','outerposition',[0 0 1 1])
hold on

input_comp = [];
input_choke = [];
for i = 1:scen
    
    % Plotting Variables
     vars_Z = {'stream4.mgdot','RUL.K','RUL.lambda','Profit.profit',...
              'CVaR.cvar','CVaR.var'...
              };
    vars_X = {'compressor.h'};  
    vars_U = {'compressor.N','choke.Z'};

    dim = size(casadi_struct2vec(V_block));
    tmp = DM(reshape(full(res.x),[res.x.size1(),scen]));
    res_split = vertsplit(tmp,dim(1));
    
    while true % Plotting
        
        % number of plots
         n_plots = length(vars_Z)+length(vars_X)+length(vars_U);
        flr = floor(sqrt(n_plots));
        cei = ceil(sqrt(n_plots));
        if flr==sqrt(n_plots)
            plot_dims = [flr,flr];
        elseif flr*cei>=n_plots
            plot_dims = [flr,cei];
        else
            plot_dims = [cei,cei];
        end

        counter = 1;

        res_Z  = {};
        res_X  = {};
        res_XC = {};
        res_U  = {};

        for r = res_split(1:end-1)
            r = full(r{:});
            r = r(:,i);
            rs = casadi_vec2struct(V_block,r(:));
            res_Z  = [res_Z  {rs.Z} ];
            res_X  = [res_X  {rs.X} ];
            res_XC = [res_XC {rs.XC}];
            res_U  = [res_U  {rs.U} ];
        end
        res_Z  = full([res_Z{:}])';
        res_X  = full([res_X{:}])';
        res_XC = full([res_XC{:}])';
        res_U  = full([res_U{:}])';

        input_comp(i)  = res_U(1,1);
        input_choke(i) = res_U(1,2);
        %% Plot Z
        [nr,nc] = size(res_Z);
        mat_Z = mat2cell(res_Z,[nr],ones(nc,1));
        indices_Z = casadi_vec2struct(dae_z,1:nc);

        t_z = [];
        tmp = N*(cumsum([0 hlist(1:end-1)]));
        for t=1:length(tmp); t_z =[t_z tmp(t)+tau_root*hlist(t)*N]; end
        t_z = t_z'/N;

        z_lb = full(z_lb_k(:,1));
        z_ub = full(z_ub_k(:,1));
        for j = 1:length(vars_Z)
            subplot(plot_dims(1),plot_dims(2),counter)
            counter = counter+1;
            hold on
            str = strsplit(vars_Z{j},'.');      
            index = full(indices_Z.(str{1}).(str{2}));
            plot(t_z,ones(size(t_z)).*z_lb(index),'r--')
            plot(t_z,ones(size(t_z)).*z_ub(index),'r--')
            plot(t_z,mat_Z{index}) 
            xlim([0,N*dt])
            title(vars_Z(j))
            xlabel('Time')
            ylabel(str{2})
        end
        %% Plot X
        
        [nr,nc] = size(res_XC);
        mat_XC = mat2cell(res_XC,[nr],ones(nc,1));
        indices_XC = casadi_vec2struct(dae_x,1:nc);

        t_xc = [];
        tmp = N*(cumsum([0 hlist(1:end-1)]));
        for t=1:length(tmp); t_xc =[t_xc tmp(t)+tau_root*hlist(t)*N]; end
        t_xc = t_xc'/N;

        xc_lb = full(x_lb_k(:,1));
        xc_ub = full(x_ub_k(:,1));
        for i = 1:length(vars_X)
            subplot(plot_dims(1),plot_dims(2),counter)
            counter = counter+1;
            hold on
            str = strsplit(vars_X{i},'.');
            index = full(indices_XC.(str{1}).(str{2}));
            plot(t_xc,ones(size(t_xc)).*xc_lb(index),'r--')
            plot(t_xc,ones(size(t_xc)).*xc_ub(index),'r--')
            plot(t_xc,mat_XC{index})
            xlim([0,N*dt])
            title(vars_X(i))
            xlabel('Time')
            ylabel(str{2})
        end
        %% Plot U
        [nr,nc] = size(res_U);
        mat_U = mat2cell(res_U,[nr],ones(nc,1));
        indices_U = casadi_vec2struct(dae_u,1:nc);

        t_u = N*(cumsum([0 hlist(1:end-1)]));
        t_u = t_u'/N;
        
        u_lb = full(u_lb_k(:,1));
        u_ub = full(u_ub_k(:,1));
        for i = 1:length(vars_U)
            subplot(plot_dims(1),plot_dims(2),counter)
            counter = counter+1;
            hold on
            str = strsplit(vars_U{i},'.');
            index = full(indices_U.(str{1}).(str{2}));
            plot(t_u,ones(size(t_u)).*u_lb(index),'r--')
            plot(t_u,ones(size(t_u)).*u_ub(index),'r--')
            stairs(t_u,mat_U{index})
            xlim([0,N*dt])
            title(vars_U(i))
            xlabel('Time')
            ylabel(str{2})
        end
        hold off
        break
    end
end