function [choke,outlet] = choke(namein,nameout,inlet,Zi)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%@Course      : Master Thesis Spring 2018
%@Task        : Function that creates a choke object for the system.
%@input       : name of the choke (namein), name of the output stream 
%               (nameout), conditions for the inlet stream (inlet), 
%               controller input for choke opening (Zi) 
%@output      : choke object (choke), outlet stream object (outlet)

%@author      : Adriaen Verheyleweghen
%@organization: Department of Chemical Engineering, NTNU, Norway
%@requires    : MATLAB R2016a (not tested in other releases)     
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    addpath('/Users/juliebergeims/downloads/casadi-matlabR2015a-v3.0.0')
    import casadi.*  
    
    nm = num2str(namein);

    outlet = def_stream(nameout,inlet.p.x,false,[],[],[],[]);
    
    % Declearing necessary variables
    Z     = MX.sym(['Z_',nm]);                         % Cchoke opening                        
    dP    = (inlet.z.P*1E7)-(outlet.z.P*1E7);          % Change in pressure
    m     = (inlet.z.mgdot*1E2)+(inlet.z.mcdot*1E2);   % Total mass flow 
    Cv    = 0.4*.4179402833086;                        % Choke constant
    
    % Creating a struct to hold the input (choke valve opening) variable
    u       = struct();
    u.Z     = Z;
    
    % Algebraic equations
    algs.m     = m - Cv*Z*sqrt(dP);                    
    algs.mgdot = outlet.z.mgdot - inlet.z.mgdot;
    algs.mcdot = outlet.z.mcdot - inlet.z.mcdot;
    algs.T     = outlet.z.T     - inlet.z.T;
    
    % Creating a choke struct
	choke         = struct();
    choke.u       = u;
    choke.algs    = algs;
    
    %% Initialize
    test = [Z-Zi;casadi_struct2vec(inlet.z)-inlet.z0];
     
    % Creating a rootfinding function to solve  
    rf_function = Function('rf_function',...
        {vertcat(casadi_struct2vec(inlet.z),...
        casadi_struct2vec(outlet.z),Z)},{vertcat(test,...
        casadi_struct2vec(outlet.algs),casadi_struct2vec(algs))});

    % Create a solver for rootfinding problem
    rf = rootfinder('rf','kinsol',rf_function);
    
    % Initial guess
    z0in = inlet.z0;
    z0out = z0in; 
    z0out(2) = z0out(2)*Zi;
    
    % Converting a CasADi matrix to a MATLAB dense matrix
    z0 = full(rf(vertcat(z0in,z0out,Zi)));
    outlet.z0 = z0(length(inlet.z0)+1:2*length(inlet.z0));
end