function [separator,outletg,outletl] = separator(namein,nameoutg,nameoutl,inlet)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%@Course      : Master Thesis Spring 2018
%@Task        : Function that creates a separator object for the system.
%@input       : name of the separator (namein), name of the gas output 
%               stream (nameoutg), name of the liquid output stream 
%               (outputl), conditions for the inlet stream (inlet) 
%@output      : separator object (separator), gas outlet stream object
%               (outletg), liquid outlet stream object (outetl)

%@author      : Adriaen Verheyleweghen
%@modified    : Julie Berge Ims
%@organization: Department of Chemical Engineering, NTNU, Norway
%@requires    : MATLAB R2016a (not tested in other releases)     
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    addpath('/Users/juliebergeims/downloads/casadi-matlabR2015a-v3.0.0')
    import casadi.*  
    
    nm = num2str(namein);
 
    % Creating streams for outlet gas and outlet liquid stream
    outletg = def_stream(nameoutg,inlet.p.x,false,[],[],[],[]);
    outletl = def_stream(nameoutl,inlet.p.x,false,[],[],[],[]);
   
    % Declare necessary variables 
    u_l_tg = MX.sym(['u_l_tg_',nm]);             % Tangential liquid vel  
    delta_l = MX.sym(['delta_l_',nm]);           % Film thickness on wall 
    alpha = MX.sym(['alpha_',nm]);               % Cyclone efficiency 
    re_ent = MX.sym(['re_ent_',nm]);             % Re-entrainment number 
    f_l_w = MX.sym(['f_l_w_',nm]);               % Frict. factor film/wall 
    f_g_i = MX.sym(['f_g_i_',nm]);               % Frict. factor gas/film
    N_my = MX.sym(['N_my_',nm]);                 % Viscosity number 
    a_l = MX.sym(['a_l_',nm]);                   % Force on liquid film 
    Re_l = MX.sym(['Re_l_',nm]);                 % Reynold nr liq. stream 
    
    % Algebraic variables 
    z       = struct();
    z.u_l_tg = u_l_tg;    
    z.theta_l = delta_l;
    z.alpha = alpha;
    z.re_ent = re_ent;
    z.f_l_w = f_l_w;
    z.f_g_i = f_g_i ;
    z.N_my = N_my;
    z.a_l = a_l;
    z. Re_l = Re_l;
    
    % Declare necessary parameters
    D = 2 * 10^(-2);                             % Inner vessel diameter 
    rho_l = 10 * 10^(2);                         % Liquid density  
    rho_g = inlet.z.rho * 10^(2);                % Gas density  
    Vgdot = (inlet.z.Vgdot)*10^(-5);             % Gas volumetric flow rate   
    
    % Declare necessary parameters for the separator
    Vldot = 0.1*Vgdot;                           % Volumetric liquid flow  
    angle = pi/4;                                % Angle film/cyclone body 
    my_l = 9.6e-8*rho_l;                         % Viscosity of liquid 
    sigma = 2.2e-3;                              % Interfacial tension 
    u_g_tg = 3.75;                               % Tangential gas velocity 
    u_g_s = 0.8* u_g_tg * tan(angle);            % Superficial gas velocity 
    
    
    % --- provided that the Re_L is in the transition regime
    %     laminar flow          2 < Re_L < 100   K = 3.73   m = -0.47
    %     transisiton regime   100< Re_L <1000   K = 1.926  m = -1/3
    
    K = 1.926;
    m = -1/3;
    
    % Model fitting parameters Austrheim(2007) 
    a_const = 0.4;
    A = -0.1345;
	B = 1.01;
  
    % Declare necessary expressions
    algs       = struct();
    
    algs.u_l_tg  = - u_l_tg+sqrt((f_g_i*1E-2)* rho_g * ((u_g_tg)^2)/    ...
                                 (f_l_w * rho_l)); 
    algs.theta_l = - (delta_l*1E05)+((cos(angle))^2) * Vldot/(pi*D*u_l_tg);
    algs.alpha   = - alpha + A * re_ent + B; 
    algs.Re_l    = - Re_l*1E2 + alpha * Vldot * rho_l * cos(angle)/     ...
                     (pi *D * my_l); 
    algs.a_l     = - a_l + (( u_l_tg)^2 )/ D/2; 
    algs.N_my    = - N_my*1E-3 + my_l/ sqrt(rho_l * sigma *             ...
                                       sqrt(sigma/(a_l * (rho_l-rho_g))) );
    algs.f_g_i   = - f_g_i*1E-2 + 0.005*(1 + 300 * (delta_l*1E05)/D/2); 
    algs.f_l_w   = - f_l_w+ (K*(Re_l*1E2)^m)^2;  
    algs.re_ent  = - re_ent + (my_l * u_g_s *((rho_g/rho_l)^0.8 )/      ...
                     (sigma * ((N_my*1E-3)^a_const) * (Re_l*1E2) ^(-1/3))); 
    
    
    algs.gas    = struct();
    algs.gas.mg = outletg.z.mgdot - inlet.z.mgdot*alpha;
    algs.gas.mc = outletg.z.mcdot - inlet.z.mcdot*(1-alpha);
    algs.gas.T  = outletg.z.T     - inlet.z.T;
    algs.gas.P  = outletg.z.P     - inlet.z.P;
    
    algs.liq    = struct();
    algs.liq.mg = outletl.z.mgdot - inlet.z.mcdot*(1-alpha);
    algs.liq.mc = outletl.z.mcdot - inlet.z.mcdot*alpha;
    algs.liq.T  = outletl.z.T     - inlet.z.T;
    algs.liq.P  = outletl.z.P     - inlet.z.P;

	separator         = struct();
    separator.z       = z;
    separator.algs    = algs;

    %% Initialize
     test = [casadi_struct2vec(inlet.z)-inlet.z0];
      
    % Vector for all variables
    vec = [ u_l_tg delta_l alpha re_ent f_l_w f_g_i N_my a_l Re_l]';
    
    % Creating a rootfinding function to solve  
    rf_function = Function('rf_function',...
        {vertcat(casadi_struct2vec(inlet.z),...        
                 casadi_struct2vec(outletg.z),...      
                 casadi_struct2vec(outletl.z),vec)},...
        {vertcat(casadi_struct2vec(outletg.algs),...   
                 casadi_struct2vec(outletl.algs),...   
                 casadi_struct2vec(algs),test)});      

    % Create a solver for rootfinding problem
    rf = rootfinder('rf','kinsol',rf_function);
    
    % Initial guess
    a=0.95;
    z0in = inlet.z0;
    z0in(9) = 0.1;
    z0outg = z0in; 
    z0outg(3) = z0outg(3)*a;
    z0outg(4) = z0outg(4)*(1-a);
    z0outg(8) = z0outg(8)*a;
    z0outg(9) = z0outg(9)*(1-a);
    
    z0outl = z0in; 
    z0outl(4) = z0outl(4)*a;
    z0outl(3) = z0outl(3)*(1-a);
    z0outl(9) = z0outl(9)*a;
    z0outl(8) = z0outl(8)*(1-a);
  
    vec_guess=[0.2205 5.2609 0.900 0.9749 0.1230 0.7 1.7 1.2155 1.655653]';
   
    % Converting a CasADi matrix to a MATLAB dense matrix
    z0 = full(rf(vertcat(z0in,z0outg,z0outl,vec_guess)));
    outletg.z0 = z0(1*length(inlet.z0)+1:2*length(inlet.z0));
    outletl.z0 = z0(2*length(inlet.z0)+1:3*length(inlet.z0));
    separator.z0 = z0(end-8:end);
    
end