function [x_struct,z_struct,u_struct,x0_struct,z0_struct,u0_struct,...
    p_struct,ode,alg]= subsea_model(u_choke, u_comp)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%@Course      : Master Thesis Spring 2018
%@Task        : Function that creates an object that defines the subsea 
%               system for with a choke, a separator and a separator.
%               Four streams connecting the units are also created.
%@input       : Choke opening (u_choke), compressor speed (u_comp)
%@output      : Differential states (x_struct), algebraic states (z_struct)
%               control inputs (u_struct), initial conditions for 
%               differential states (x0_struct, initial conditions for
%               alfebraic states (z0_struct), inital values for inputs
%               (u0_struct), parameters (p_struct), differential eguations
%               (ode), algebraic equations (alg)

%@author      : Adriaen Verheyleweghen
%@modified    : Julie Berge Ims
%@organization: Department of Chemical Engineering, NTNU, Norway
%@requires    : MATLAB R2016a (not tested in other releases)     
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

   
    % Link to Casadi installation folder
    addpath('/Users/juliebergeims/downloads/casadi-matlabR2015a-v3.0.0')
    import casadi.*
    addpath([pwd,'/functions'])
    
    % Declare necessary parameters for stream 1
    x = [.92,.05,.02,.005,.005,0,0,0,0,0,0];    % Composition of the fluid
    T1 = 3.5;                                   % Inlet temperature 350 K
    P1 = 1.0;                                   % Inlet pressure 100 bar
    mgdot1 = 0.9;                               % Inlet gas mass flow rate
    mcdot1 = 0;                                 % Inlet liquid mass flow 
    
    % Declare other necessary parameters for the process
    comp_0 = 0.01;                              % Init. comp. bearing degr.
    

    % First well: def_stream(name,x,init,Ti,Pi,mgdoti,mcdoti)
    stream1 = def_stream('1',x,true,T1,P1,mgdot1,mcdot1)  

    % Choke: choke(namein,nameout,inlet,Zi)
    [chk,stream2] = choke('chk','2',stream1,u_choke);                      
    
    % Separator: separator(namein,nameoutg,nameoutl,inlet)
    [sep,stream3,stream5] = separator('sep','3','5',stream2);
    
    % Compressor: compressor(namein,nameout,inlet,Ni,hi)
    [comp,stream4] = compressor('comp34','4',stream3,u_comp,comp_0);
    
    %% Concatenate the decision variables
    
    % Algebraic variables
    z_struct            = struct();
    
    % Remove unneeded variables (to reduce size of system)
    z_struct.stream1    = rmfield(stream1.z,                            ...
                                {'Cp'});
    z_struct.stream2    = rmfield(stream2.z,                            ...
                                {'Cp','Vcdot','GVF'});
    z_struct.stream3    = rmfield(stream3.z,                            ...
                                {});
    z_struct.stream4    = rmfield(stream4.z,                            ...
                                {'rho','Vgdot','Vcdot','Z','GVF'});
    z_struct.stream5    = rmfield(stream5.z,                            ...
                                {'Cp','rho','Vgdot','Vcdot','Z','GVF'});
    z_struct.separator  = sep.z;
    z_struct.compressor = comp.z;
    
    % Same for the struct of initial conditions
    z0_struct            = struct();
    z0_struct.stream1    = rmfield(casadi_vec2struct(stream1.z,         ...
                                 stream1.z0),                           ...
                                 {'Cp'}                                 ...
                                 );
    z0_struct.stream2    = rmfield(casadi_vec2struct(stream2.z,         ...
                                 stream2.z0),                           ...
                                 {'Cp','Vcdot','GVF'}                   ...
                                 );
    z0_struct.stream3    = rmfield(casadi_vec2struct(stream3.z,         ...
                                stream3.z0),                            ...
                                {}                                      ...
                                );
    z0_struct.stream4    = rmfield(casadi_vec2struct(stream4.z,         ...
                                stream4.z0),                            ...
                                {'rho','Vgdot','Vcdot','Z','GVF'}       ...
                                );
    z0_struct.stream5    = rmfield(casadi_vec2struct(stream5.z,         ...
                                stream5.z0),                            ...
                                {'Cp','rho','Vgdot','Vcdot','Z','GVF'}  ...
                                );
    z0_struct.separator  = sep.z0;
    z0_struct.compressor = comp.z0;

    % Differential variables
    x_struct = struct();
    x_struct.compressor = comp.x;
    
    x0_struct = struct();
    x0_struct.compressor = comp_0;

    % Inputs
    u_struct = struct();
    u_struct.compressor = comp.u;
    u_struct.choke      = chk.u;
    
    u0_struct = struct();
    u0_struct.compressor = u_comp;
    u0_struct.choke      = u_choke;
    
    % Parameters
    p_struct = struct();
    p_struct.a = comp.p.a;
    p_struct.GVF = stream1.z.GVF;
    
    %% Define equations

    % Algebraic equations
    sep_algs = sep.algs;
    sep_algs.gas = rmfield(sep_algs.gas,{'T','P'});
    sep_algs.liq = rmfield(sep_algs.liq,{'T','P'});

    alg = [                                                             ...
           casadi_struct2vec(rmfield(stream1.algs,                      ...
                             {'Cp'}                                     ...
                             ));                                        ...
           casadi_struct2vec(rmfield(stream2.algs,                      ...
                             {'Cp','Vcdot','GVF'}                       ...
                             ));                                        ...
           casadi_struct2vec(rmfield(stream3.algs,                      ...
                             {}                                         ...
                             ));                                        ...
           casadi_struct2vec(rmfield(stream4.algs,                      ...
                             {'rho','Vgdot','Vcdot','Z','GVF'}          ...
                             ));                                        ...
           casadi_struct2vec(rmfield(stream5.algs,                      ...
                             {'Cp','rho','Vgdot','Vcdot','Z','GVF'}     ...
                             ));                                        ...
           casadi_struct2vec(rmfield(chk.algs,                          ...
                             {'T'}                                      ...
                             ));                                        ...
           casadi_struct2vec(sep_algs);                                 ...
           casadi_struct2vec(rmfield(comp.algs,{}));                    ...
          ];

    % Differential equations
    ode = [                                                             ...
           casadi_struct2vec(comp.odes);                                ...
          ];
    
    % Boundary conditions  
    s1 = {...                                         
          {'stream1','T'},                                              ...
          {'stream2','T'},                                              ...
          {'stream3','T'},                                              ... 
          {'stream5','T'},                                              ... 
          {'stream1','P'},                                              ...
          {'stream2','P'},                                              ...
          {'stream3','P'},                                              ...
          {'stream5','P'},                                              ...
         };                         

    v1 = {...
          3.5,                                                          ...
          3.5,                                                          ...
          3.5,                                                          ...
          3.5,                                                          ...
          1.0,                                                          ...
           .9,                                                          ...
           .9,                                                          ...
           .9,                                                          ...
         };

    for i=1:length(s1)
        cell = s1{i};
        z                   = z_struct.(cell{1}).(cell{2});
        z_struct.(cell{1})  = rmfield(z_struct.(cell{1}),cell{2});
        z0_struct.(cell{1}) = rmfield(z0_struct.(cell{1}),cell{2});
        alg = substitute(alg ,z,v1{i});
        ode = substitute(ode ,z,v1{i});
    end
end